import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';

type useIsAuthorizedState = {
  authorized: boolean | undefined;
  jwt: string | undefined;
  sessionID: string | undefined;
  userID: string | undefined;
};

// Define the initial state using that type
export const initialState: useIsAuthorizedState = {
  authorized: undefined,
  jwt: undefined,
  sessionID: undefined,
  userID: undefined,
};

export const authSlice = createSlice({
  name: 'auth',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    authorized(state, action: PayloadAction<useIsAuthorizedState>) {
      console.debug('Authorized');
      state.authorized = action.payload.authorized;
      state.jwt = action.payload.jwt;
      state.sessionID = action.payload.sessionID;
      state.userID = action.payload.userID;
    },
    logout(state) {
      console.debug('Logging out');
      state.authorized = undefined;
      state.jwt = undefined;
      state.sessionID = undefined;
      state.userID = undefined;
    },
    unauthorized(state) {
      console.debug('Unauthorized');
      state.authorized = false;
    },
  },
});

export const { authorized, unauthorized, logout } = authSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const authState = (state: RootState) => state.auth;

export default authSlice.reducer;
